/*
 * Copyright (c) 2005-2006 Laf-Widget Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Laf-Widget Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.lafwidget.utils;

import org.jvnet.lafwidget.utils.FadeTracker.FadeKind;

/**
 * Implementation of {@link FadeStep} for accelerated fades.
 * 
 * @author Kirill Grouchnikov
 */
public class FadeAccelerationStep implements FadeStep {
	/**
	 * Delegate.
	 */
	protected FadeStep delegate;

	/**
	 * The acceleration factor.
	 */
	protected float accelerationFactor;

	/**
	 * Creates an accelerated implementation.
	 * 
	 * @param delegate
	 *            Delegate.
	 * @param accelerationFactor
	 *            Acceleration factor.
	 */
	public FadeAccelerationStep(FadeStep delegate, float accelerationFactor) {
		this.delegate = delegate;
		this.accelerationFactor = accelerationFactor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.lafwidget.utils.FadeStep#getNextStep(org.jvnet.lafwidget.utils.FadeTracker.FadeKind,
	 *      float, boolean, boolean)
	 */
	public float getNextStep(FadeKind fadeKind, float currFadePosition,
			boolean isFadeIn, boolean isLooping) {
		return this.delegate.getNextStep(fadeKind, currFadePosition, isFadeIn,
				isLooping)
				* this.accelerationFactor;
	}
}